@@ -6,6 +6,8 @@ using System.Runtime.InteropServices;
 
 namespace FSpot {
 public class ImageView : Layout {
+	private Cms.Transform transform;
+
 	[DllImport ("libfspot")]
 	static extern IntPtr f_image_view_new ();
 
@@ -135,6 +137,24 @@ public class ImageView : Layout {
 						 out x_offset, out y_offset, out scaled_width, out scaled_height);
 	}
 
+	public Gdk.Rectangle ImageCoordsToWindow (Gdk.Rectangle image)
+	{
+		int x, y;
+		int width, height;
+
+		if (this.Pixbuf == null)
+			return Gdk.Rectangle.Zero;
+		
+		this.GetOffsets (out x, out y, out width, out height);
+
+		Gdk.Rectangle win = Gdk.Rectangle.Zero;
+		win.X = (int) Math.Floor (image.X * (double) (width - 1) / (this.Pixbuf.Width - 1) + 0.5) + x;
+		win.Y = (int) Math.Floor (image.Y * (double) (height - 1) / (this.Pixbuf.Height - 1) + 0.5) + y;
+		win.Width = (int) Math.Floor ((image.X + image.Width) * (double) (width - 1) / (this.Pixbuf.Width - 1) + 0.5) - win.X + x;
+		win.Height = (int) Math.Floor ((image.Y + image.Height) * (double) (height - 1) / (this.Pixbuf.Height - 1) + 0.5) - win.Y + y;
+
+		return win;
+	}
 
 	[DllImport ("libfspoteog")]
 	static extern void image_view_set_display_brightness (IntPtr view, float display_brightness);
@@ -157,6 +177,20 @@ public class ImageView : Layout {
 		}
 	}
 
+	[DllImport ("libfspoteog")]
+	static extern void image_view_set_display_transform (IntPtr view, IntPtr transform);
+
+	public Cms.Transform Transform {
+		set {
+			this.transform = value;
+			image_view_set_display_transform (Handle, transform.Handle);
+		}
+		get {
+			return transform;
+		}
+	}
+
+
 	private delegate void SelectionChangedDelegate (IntPtr obj, IntPtr data);
 	private static void SelectionChangedCallback (IntPtr raw, IntPtr unused_data)
 	{
