@@ -4,7 +4,6 @@ using GtkSharp;
 using Glade;
 using Gnome;
 using System;
-using System.IO;
 using System.Text;
 
 using System.Collections;
@@ -65,7 +64,8 @@ public class MainWindow {
 	FSpot.InfoDisplay info_display;
 	QueryView icon_view;
 	PhotoView photo_view;
-	PhotoQuery query;
+	FSpot.FullScreenView fsview;
+	FSpot.PhotoQuery query;
 	FSpot.GroupSelector group_selector;
 	
 	// Drag and Drop
@@ -94,25 +94,44 @@ public class MainWindow {
 		new TargetEntry ("text/uri-list", 0, (uint) TargetType.UriList),
 	};
 
-	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
 	const int PHOTO_IDX_NONE = -1;
-	private int current_photo_idx = PHOTO_IDX_NONE;
-	private bool current_photos = false;
 	
 	struct YearCount {
 		int Year;
 		int Count;
 	}
 
+	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
+	private int ActiveIndex () {
+	        int [] selection = SelectedIds ();
+		
+		if (selection.Length == 1) 
+			return selection [0];
+		else 
+			return PHOTO_IDX_NONE;
+	}
+
+	public bool PhotoSelectionActive ()
+	{
+		return SelectedIds().Length > 0;
+	}
+
 	private Photo CurrentPhoto {
 		get {
-			if (current_photo_idx != PHOTO_IDX_NONE)
-				return query.Photos [current_photo_idx];
+			int active = ActiveIndex ();
+			if (active >= 0)
+				return query.Photos [active];
 			else
 				return null;
 		}
 	}
 
+	// Switching mode.
+	enum ModeType {
+		IconView,
+		PhotoView
+	};
+	ModeType mode;
 	
 	//
 	// Constructor
@@ -120,7 +139,8 @@ public class MainWindow {
 	public MainWindow (Db db)
 	{
 		this.db = db;
-		
+
+		Mono.Posix.Catalog.Init ("f-spot", Defines.LOCALE_DIR);
 		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", null);
 		gui.Autoconnect (this);
 
@@ -144,7 +164,7 @@ public class MainWindow {
 		info_box.VersionIdChanged += new InfoBox.VersionIdChangedHandler (HandleInfoBoxVersionIdChange);
 		left_vbox.PackStart (info_box, false, true, 0);
 		
-		query = new PhotoQuery (db.Photos);
+		query = new FSpot.PhotoQuery (db.Photos);
 		query.ItemChanged += HandleQueryItemChanged;
 
 		group_selector = new FSpot.GroupSelector ();
@@ -192,10 +212,11 @@ public class MainWindow {
 
 		photo_view = new PhotoView (query, db.Photos);
 		photo_box.Add (photo_view);
-		photo_view.PhotoChanged += new PhotoView.PhotoChangedHandler (HandlePhotoViewPhotoChanged);
-		photo_view.ButtonPressEvent += new ButtonPressEventHandler (HandlePhotoViewButtonPressEvent);
-		photo_view.UpdateStarted += new PhotoView.UpdateStartedHandler (HandlePhotoViewUpdateStarted);
-		photo_view.UpdateFinished += new PhotoView.UpdateFinishedHandler (HandlePhotoViewUpdateFinished);
+		photo_view.PhotoChanged += HandlePhotoViewPhotoChanged;
+		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
+		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
+		photo_view.UpdateStarted += HandlePhotoViewUpdateStarted;
+		photo_view.UpdateFinished += HandlePhotoViewUpdateFinished;
 
 		Gtk.Drag.DestSet (photo_view, DestDefaults.All, tag_target_table, 
 				  DragAction.Copy | DragAction.Move); 
@@ -214,18 +235,45 @@ public class MainWindow {
 			Toplevel = this;
 	}
 
-	
+	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
+	{
+		switch (view_notebook.CurrentPage) {
+		case 0:
+			mode = ModeType.IconView;
+			Present (photo_view.CurrentPhoto);
+			break;
+		case 1:
+			mode = ModeType.PhotoView;
+			Present (icon_view.FocusCell);
+			break;
+		}
+	}
+
+	void SwitchToIconViewMode ()
+	{
+		view_notebook.CurrentPage = 0;
+	}
+
+	void SwitchToPhotoViewMode ()
+	{
+		view_notebook.CurrentPage = 1;
+	}
+
 	public int [] SelectedIds () {
 		int [] ids;
-		switch (mode) {
-		case ModeType.IconView:
-			ids = icon_view.SelectedIdxs;
-			break;
-		default:
-		case ModeType.PhotoView:
-			ids = new int [1];
-			ids [0] = photo_view.CurrentPhoto;
-			break;
+
+		if (fsview != null)
+			ids = new int [] { fsview.View.CurrentPhoto };
+		else {
+			switch (mode) {
+			case ModeType.IconView:
+				ids = icon_view.SelectedIdxs;
+				break;
+			default:
+			case ModeType.PhotoView:
+				ids = new int [] { photo_view.CurrentPhoto };
+				break;
+			}
 		}
 
 		return ids;
@@ -252,7 +300,6 @@ public class MainWindow {
 		return SelectedPhotos (SelectedIds ());
 	}
 
-
 	//
 	// Change Notification functions
 	//
@@ -260,16 +307,9 @@ public class MainWindow {
 	private void InvalidateViews ()
 	{
 		icon_view.QueueDraw ();
-		photo_view.Update ();
-	}
-
-	private void UpdateViews (int num)
-	{
-		icon_view.UpdateThumbnail (num);
-		if (num == photo_view.CurrentPhoto)
-			photo_view.Update ();
-		info_box.Update ();
-		UpdateMenus ();
+		photo_view.Reload ();
+		if (fsview != null)
+			fsview.View.Reload ();
 	}
 		
 	//
@@ -284,7 +324,7 @@ public class MainWindow {
 		int [] selected_ids = SelectedIds ();
 		if (command.Execute (direction, SelectedPhotos (selected_ids))) {
 			foreach (int num in selected_ids)
-				UpdateViews (num);
+				query.MarkChanged (num);
 		}
 	}
 
@@ -314,8 +354,6 @@ public class MainWindow {
 		}
 	}
 
-
-
 	[GLib.ConnectBefore]
 	void HandleTagSelectionButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
@@ -408,7 +446,7 @@ public class MainWindow {
 		Photo [] photos = SelectedPhotos ();
 		
 		if (photos.Length > 0) {
-			string thumbnail_path = Thumbnail.PathForUri ("file://" + photos[0].DefaultVersionPath, ThumbnailSize.Large);
+			string thumbnail_path = Thumbnail.PathForUri (photos[0].DefaultVersionUri.ToString (), ThumbnailSize.Large);
 			Pixbuf thumbnail = ThumbnailCache.Default.GetThumbnailForPath (thumbnail_path);
 			if (thumbnail != null) {
 				Gtk.Drag.SetIconPixbuf (args.Context, thumbnail, 0, 0);
@@ -456,8 +494,6 @@ public class MainWindow {
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
-		
-		//Console.WriteLine ("IconView View Drag received {0} type {1}", source == null ? "null" : source.TypeName, (TargetType)args.Info);
 
 		switch (args.Info) {
 		case (uint)TargetType.TagList:
@@ -498,23 +534,6 @@ public class MainWindow {
 
 	void HandleSelectionChanged (IconView view)
 	{
-		int [] selection = SelectedIds ();
-
-		if (selection.Length == 1) {
-			current_photo_idx = selection [0];
-			//photo_view.CurrentPhoto = current_photo_idx;
-			info_box.Photo = CurrentPhoto;
-			if (info_display != null) 
-				info_display.Photo = CurrentPhoto;
-		} else { 
-			current_photo_idx = PHOTO_IDX_NONE;
-			info_box.Photo = null;
-			if (info_display != null)
-				info_display.Photo = null;
-		}
-
-		current_photos = selection.Length > 0;
-			
 		UpdateMenus ();
 	}
 
@@ -530,15 +549,27 @@ public class MainWindow {
 
 	void HandlePhotoViewPhotoChanged (PhotoView sender)
 	{
-		current_photos = true;
-		current_photo_idx = photo_view.CurrentPhoto;
 		info_box.Photo = CurrentPhoto;
 		if (info_display != null)
 			info_display.Photo = CurrentPhoto;
 		UpdateMenus ();
 	}
+	
+	void HandlePhotoViewKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
+	{
+		switch (args.Event.Key) {
+		case Gdk.Key.F:
+		case Gdk.Key.f:
+			HandleViewFullscreen (sender, args);
+			args.RetVal = true;
+			break;
+		default:
+			break;
+		}
+		return;
+	}
 
-	void HandlePhotoViewButtonPressEvent (object sender, ButtonPressEventArgs args)
+	void HandlePhotoViewButtonPressEvent (object sender, Gtk.ButtonPressEventArgs args)
 	{
 		if (args.Event.Type == EventType.TwoButtonPress && args.Event.Button == 1)
 			SwitchToIconViewMode ();
@@ -741,33 +772,25 @@ public class MainWindow {
 	}
 
 
-#if false
-	void HandleExportCommand (object sender, EventArgs args)
+	void HandleExportToGallery (object sender, EventArgs args)
 	{
-		ExportCommand.Gallery cmd = new ExportCommand.Gallery ();
-
-		if (cmd.Execute (SelectedPhotos ())) {
-			Console.WriteLine ("success");
-		}
-	}	
-#else
-	void HandleExportCommand (object sender, EventArgs args)
-	{
-		FSpot.GalleryExport export = new FSpot.GalleryExport (SelectedPhotos ());
+		FSpot.GalleryExport export = new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
-#endif
 
-#if false 
-	void HandleExportCommand (object sender, EventArgs args)
+	void HandleExportToVfs (object sender, EventArgs args)
 	{
-		FotkiRemote fr = new FotkiRemote ();
-		//fr.Login ("mikey@gmail.com", "joey");
-		fr.Login ("mikey", "joey");
-		foreach (Photo p in SelectedPhotos ()) {
-			fr.Upload (p);
-		}
+		FSpot.VfsExport export = new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
+
+	void HandleExportToFlickr (object sender, EventArgs args)
+	{
+		FSpot.FlickrExport export = new FSpot.FlickrExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
+	
+	void HandleExportToFotki (object sender, EventArgs args)
+	{
+		
 	}
-#endif
 
 	void HandleSendMailCommand (object sender, EventArgs args)
 	{
@@ -799,12 +822,19 @@ public class MainWindow {
 			"Grahm Orr",
 			"Cheslack-Postava",
 			"Patanjali Somayaji",
-			"Matt Jones"
-		};
-			
-		Gnome.About about = new About ("F-Spot", "0.0.3", "Copyright 2003-2004 Novell Inc.", null, authors, null, null, null);
+			"Matt Jones",
+			"Martin Willemoes Hansen",
+			"Laurence Hygate"
 
-		about.Show ();
+		};
+
+                // Translators should localize the following string
+                // * which will give them credit in the About box.
+                // * E.g. "Martin Willemoes Hansen"
+                string translators = Mono.Posix.Catalog.GetString ("translator-credits");
+
+                new About ("F-Spot", "0.0.3", "Copyright 2003-2004 Novell Inc.",
+                           null, authors, null, translators, null).Show();
 	}
 
 	void HandleArrangeByTime (object sender, EventArgs args)
@@ -838,7 +868,7 @@ public class MainWindow {
 		PhotoVersionCommands.Create cmd = new PhotoVersionCommands.Create ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			UpdateViews (current_photo_idx);
+			query.MarkChanged (ActiveIndex ());
 		}
 	}
 
@@ -847,7 +877,7 @@ public class MainWindow {
 		PhotoVersionCommands.Delete cmd = new PhotoVersionCommands.Delete ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			UpdateViews (current_photo_idx);
+			query.MarkChanged (ActiveIndex ());
 		}
 	}
 
@@ -858,7 +888,7 @@ public class MainWindow {
 	        long length = 0;
 
 		foreach (Photo p in photos) {
-			FileInfo fi = new FileInfo (p.DefaultVersionPath);
+			System.IO.FileInfo fi = new System.IO.FileInfo (p.DefaultVersionPath);
 
 			length += fi.Length;
 		}
@@ -871,7 +901,7 @@ public class MainWindow {
 		PhotoVersionCommands.Rename cmd = new PhotoVersionCommands.Rename ();
 
 		if (cmd.Execute (db.Photos, CurrentPhoto, main_window)) {
-			UpdateViews (current_photo_idx);
+			query.MarkChanged (ActiveIndex ());
 		}
 	}
 
@@ -924,6 +954,75 @@ public class MainWindow {
 			tag_selection_widget.Update ();
 	}
 
+	void HandleAdjustColor (object sender, EventArgs args)
+	{
+		if (ActiveIndex () > 0) {
+			new FSpot.ColorDialog (query, ActiveIndex ());
+		}
+	}
+
+	void HandleSharpen (object sender, EventArgs args)
+	{
+		Gtk.Dialog dialog = new Gtk.Dialog (Mono.Posix.Catalog.GetString ("Unsharp Mask"), main_window, Gtk.DialogFlags.Modal);
+
+		dialog.VBox.Spacing = 6;
+		dialog.VBox.BorderWidth = 12;
+		
+		Gtk.Table table = new Gtk.Table (3, 2, false);
+		table.ColumnSpacing = 6;
+		table.RowSpacing = 6;
+		
+		table.Attach (new Gtk.Label (Mono.Posix.Catalog.GetString ("Amount:")), 0, 1, 0, 1);
+		table.Attach (new Gtk.Label (Mono.Posix.Catalog.GetString ("Radius:")), 0, 1, 1, 2);
+		table.Attach (new Gtk.Label (Mono.Posix.Catalog.GetString ("Threshold:")), 0, 1, 2, 3);
+
+		Gtk.SpinButton amount_spin = new Gtk.SpinButton (0.0, 100.0, .01);
+		Gtk.SpinButton radius_spin = new Gtk.SpinButton (0.0, 50.0, .01);
+		Gtk.SpinButton threshold_spin = new Gtk.SpinButton (0.0, 50.0, .01);
+
+		table.Attach (amount_spin, 1, 2, 0, 1);
+		table.Attach (radius_spin, 1, 2, 1, 2);
+		table.Attach (threshold_spin, 1, 2, 2, 3);
+		
+		dialog.VBox.PackStart (table);
+
+		dialog.AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel);
+		dialog.AddButton (Gtk.Stock.Ok, Gtk.ResponseType.Ok);
+
+		dialog.ShowAll ();
+
+		Gtk.ResponseType response = (Gtk.ResponseType) dialog.Run ();
+
+		if (response == Gtk.ResponseType.Ok) {
+			foreach (int id in SelectedIds ()) {
+				Gdk.Pixbuf orig = FSpot.PhotoLoader.Load (query, id);
+				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
+			
+				Photo photo = query.Photos [id];
+
+				uint version = photo.DefaultVersionId;
+				if (version == Photo.OriginalVersionId) {
+					version = photo.CreateDefaultModifiedVersion (photo.DefaultVersionId, false);
+				}
+				
+				try {
+					string version_path = photo.GetVersionPath (version);
+					
+					final.Savev (version_path, "jpeg", null, null);
+					PhotoStore.GenerateThumbnail (version_path);
+					photo.DefaultVersionId = version;
+					query.Commit (id);
+				} catch (GLib.GException ex) {
+					// FIXME error dialog.
+					Console.WriteLine ("error {0}", ex);
+				}
+			
+			}
+		}
+
+		dialog.Destroy ();
+	}
+
 	void HandleViewSmall (object sender, EventArgs args)
 	{
 		icon_view.ThumbnailWidth = 64;	
@@ -1025,37 +1124,50 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
-	FSpot.FullScreenView fsview;
 	void HandleViewFullscreen (object sender, EventArgs args)
 	{
-		fsview = new FSpot.FullScreenView (query);
-		fsview.Show ();
-
+		int active = Math.Max (ActiveIndex (), 0);
+		if (fsview == null) {
+			fsview = new FSpot.FullScreenView (query);
+			fsview.Destroyed += HandleFullScreenViewDestroy;
+		}
 		// FIXME this needs to be another mode like PhotoView and IconView mode.
+		fsview.View.CurrentPhoto = active;
 
-		if (current_photo_idx != PHOTO_IDX_NONE)
-			fsview.View.CurrentPhoto = current_photo_idx;
-		else 
-			fsview.View.CurrentPhoto = 0;
-
-
+		fsview.Show ();
+	}
+	
+	void Present (int item)
+	{
+		switch (mode) {
+		case ModeType.IconView:
+			icon_view.ScrollTo (item);
+			icon_view.Throb (item);
+			break;
+		case ModeType.PhotoView:
+			photo_view.CurrentPhoto = item;
+			break;
+		}
 	}
 
+	void HandleFullScreenViewDestroy (object sender, EventArgs args)
+	{
+		Present (fsview.View.CurrentPhoto);
+		fsview = null;
+	}
+	
 	void HandleZoomOut (object sender, EventArgs args)
 	{
 		switch (mode) {
 		case ModeType.PhotoView:
 			double old_zoom = photo_view.Zoom;
-			try {
-				photo_view.Zoom -= .1;
-			} catch {
-				if (old_zoom - .1 < -0.09) {
-					photo_view.Zoom = 0.0;
-					SwitchToIconViewMode ();
-				}
+
+			old_zoom /= FSpot.PhotoImageView.ZoomMultipler;
+			if (old_zoom < .001) {
+				SwitchToIconViewMode ();
+			} else {
 				photo_view.Zoom = old_zoom;
 			}
-			
 			break;
 		case ModeType.IconView:
 			int width = icon_view.ThumbnailWidth;
@@ -1075,7 +1187,7 @@ public class MainWindow {
 		case ModeType.PhotoView:
 			double old_zoom = photo_view.Zoom;
 			try {
-				photo_view.Zoom += .1;
+				photo_view.Zoom *= FSpot.PhotoImageView.ZoomMultipler;
 			} catch {
 				photo_view.Zoom = old_zoom;
 			}
@@ -1145,7 +1257,7 @@ public class MainWindow {
 		int [] selected_ids = SelectedIds ();
 		if (command.Execute (SelectedPhotos (selected_ids))) {
 			foreach (int num in selected_ids)
-				UpdateViews (num);
+				query.MarkChanged (num);
 		}
 	}
 
@@ -1214,9 +1326,9 @@ public class MainWindow {
 	void UpdateForVersionIdChange (uint version_id)
 	{
 		CurrentPhoto.DefaultVersionId = version_id;
-		query.Commit (current_photo_idx);
+		int active = ActiveIndex ();
 
-		UpdateViews (current_photo_idx);
+		query.Commit (active);
 	}
 
 	void HandleVersionIdChanged (PhotoVersionMenu menu)
@@ -1251,7 +1363,7 @@ public class MainWindow {
 		UpdateMenus ();
 	}
 
-	void HandleQueryItemChanged (PhotoQuery query, int item)
+	void HandleQueryItemChanged (FSpot.IPhotoCollection query, int item)
 	{
 		Photo photo = query.Photos [item];
 		if (info_box.Photo == photo)
@@ -1291,51 +1403,18 @@ public class MainWindow {
 		}
 
 		bool tag_sensitive = tag_selection_widget.Selection.CountSelectedRows () > 0;
-		
-		rotate_left.Sensitive = current_photos;
-		rotate_right.Sensitive = current_photos;
-		update_thumbnail.Sensitive = current_photos;
-		delete_from_drive.Sensitive = current_photos;
-		copy.Sensitive = current_photos;
+		bool active_selection = PhotoSelectionActive ();
+
+		rotate_left.Sensitive = active_selection;
+		rotate_right.Sensitive = active_selection;
+		update_thumbnail.Sensitive = active_selection;
+		delete_from_drive.Sensitive = active_selection;
+		copy.Sensitive = active_selection;
 
 		delete_selected_tag.Sensitive = tag_sensitive;
-		attach_tag_to_selection.Sensitive = tag_sensitive && current_photos;
-		remove_tag_from_selection.Sensitive = tag_sensitive && current_photos;
+		attach_tag_to_selection.Sensitive = tag_sensitive && active_selection;
+		remove_tag_from_selection.Sensitive = tag_sensitive && active_selection;
 	}
 
-	// Switching mode.
-
-	enum ModeType {
-		IconView,
-		PhotoView
-	};
-	ModeType mode;
-
-	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
-	{
-		switch (view_notebook.CurrentPage) {
-		case 0:
-			icon_view.ScrollTo (photo_view.CurrentPhoto);
-			icon_view.Throb (photo_view.CurrentPhoto);
-
-			mode = ModeType.IconView;
-			break;
-		case 1:
-			photo_view.CurrentPhoto = icon_view.FocusCell;
-
-			mode = ModeType.PhotoView;
-			break;
-		}
-	}
-
-	void SwitchToIconViewMode ()
-	{
-		view_notebook.CurrentPage = 0;
-	}
-
-	void SwitchToPhotoViewMode ()
-	{
-		view_notebook.CurrentPage = 1;
-	}
 }
 
