@@ -21,24 +21,24 @@ public class PhotoPopup {
 		int count = MainWindow.Toplevel.SelectedIds ().Length;
 		
 		Gtk.Menu popup_menu = new Gtk.Menu ();
-		bool have_selection = count != 0;
+		bool have_selection = count > 0;
 		bool have_multi = count > 1;
 
-		GtkUtil.MakeMenuItem (popup_menu, "Copy Image Location", 
+		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Copy Image Location"), 
 				      new EventHandler (MainWindow.Toplevel.HandleCopyLocation), have_selection);
 		
 		GtkUtil.MakeMenuSeparator (popup_menu);
 
-		GtkUtil.MakeMenuItem (popup_menu, "Rotate Left", "f-spot-rotate-270",
+		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Rotate Left"), "f-spot-rotate-270",
 				      new EventHandler (MainWindow.Toplevel.HandleRotate270Command), have_selection);
-		GtkUtil.MakeMenuItem (popup_menu, "Rotate Right", "f-spot-rotate-90",
+		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Rotate Right"), "f-spot-rotate-90",
 				      new EventHandler (MainWindow.Toplevel.HandleRotate90Command), have_selection);
 
 		GtkUtil.MakeMenuSeparator (popup_menu);
 
-		GtkUtil.MakeMenuItem (popup_menu, "Remove From Catalog", 
+		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Remove From Catalog"), 
 				      new EventHandler (MainWindow.Toplevel.HandleRemoveCommand), have_selection);
-		GtkUtil.MakeMenuItem (popup_menu, "Delete From Drive",
+		GtkUtil.MakeMenuItem (popup_menu, Mono.Posix.Catalog.GetString ("Delete From Drive"),
 				      new EventHandler (MainWindow.Toplevel.HandleDeleteCommand), have_selection);
 
 		GtkUtil.MakeMenuSeparator (popup_menu);
@@ -46,7 +46,7 @@ public class PhotoPopup {
 		//
 		// FIXME TagMenu is ugly.
 		//
-		MenuItem attach_item = new MenuItem ("Attach Tag");
+		MenuItem attach_item = new MenuItem (Mono.Posix.Catalog.GetString ("Attach Tag"));
 		TagMenu attach_menu = new TagMenu (attach_item, MainWindow.Toplevel.Database.Tags);
 		attach_menu.TagSelected += MainWindow.Toplevel.HandleAttachTagMenuSelected;
 		attach_item.ShowAll ();
@@ -56,7 +56,7 @@ public class PhotoPopup {
 		// FIXME finish the IPhotoSelection stuff and move the activate handler into the class
 		// this current method is way too complicated.
 		//
-		MenuItem remove_item = new MenuItem ("Remove Tag");
+		MenuItem remove_item = new MenuItem (Mono.Posix.Catalog.GetString ("Remove Tag"));
 		PhotoTagMenu remove_menu = new PhotoTagMenu ();
 		remove_menu.TagSelected += MainWindow.Toplevel.HandleRemoveTagMenuSelected;
 		remove_item.Submenu = remove_menu;
