@@ -6,23 +6,17 @@ using System;
 
 public class PhotoView : EventBox {
 
-	private int current_photo;
 	public int CurrentPhoto {
 		get {
-			return current_photo;
+			return photo_view.CurrentPhoto;
 		}
-
 		set {
-			if (current_photo == value)
-				return;
-
-			current_photo = value;
-			Update ();
+			photo_view.CurrentPhoto = value;
 		}
 	}
 
 	private bool CurrentPhotoValid () {
-		if (query == null || query.Photos.Length == 0 || current_photo >= Query.Photos.Length)
+		if (query == null || query.Photos.Length == 0 || CurrentPhoto >= Query.Photos.Length)
 			return false;
 
 		return true;
@@ -30,35 +24,24 @@ public class PhotoView : EventBox {
 
 	private PhotoStore photo_store;
 
-	private PhotoQuery query;
-	public PhotoQuery Query {
+	private FSpot.PhotoQuery query;
+	public FSpot.PhotoQuery Query {
 		get {
 			return query;
 		}
 
 		set {
 			query = value;
-			query.Reload += HandleQueryReload;
-			query.ItemChanged += HandleQueryItemChanged;
-
-			// FIXME which picture to display?
-			current_photo = 0;
-			Update ();
 		}
 	}
 	
-	private void HandleQueryReload (PhotoQuery query) 
+	public void Reload ()
 	{
-		Update ();
-	}
-	
-	private void HandleQueryItemChanged (PhotoQuery query, int item)
-	{
-		if (item == current_photo)
-			Update ();
+		photo_view.Reload ();
+
 	}
 
-	private FSpot.ImageView image_view;
+	private FSpot.PhotoImageView photo_view;
 	private TagView tag_view;
 	private Button display_next_button, display_previous_button;
 	private Label count_label;
@@ -68,18 +51,13 @@ public class PhotoView : EventBox {
 
 	private const double MAX_ZOOM = 5.0;
 
-	private double zoom;
 	public double Zoom {
 		get {
-			return zoom;
+			return photo_view.Zoom;
 		}
 
 		set {
-			if ((value < 0.0) || (value > MAX_ZOOM))
-				throw new Exception (String.Format ("Zoom value out of range {0}", value));
-				
-			zoom = value;
-			UpdateZoom ();
+			photo_view.Zoom = value;
 		}
 	}
 
@@ -111,7 +89,7 @@ public class PhotoView : EventBox {
 	private void HandleSelectionConstraintOptionMenuActivated (object sender, EventArgs args)
 	{
 		selection_constraint_ratio_idx = (int) (sender as GLib.Object).Data [CONSTRAINT_RATIO_IDX_KEY];
-		image_view.SelectionXyRatio = constraints [selection_constraint_ratio_idx].XyRatio;
+		photo_view.SelectionXyRatio = constraints [selection_constraint_ratio_idx].XyRatio;
 	}
 
 	private OptionMenu CreateConstraintsOptionMenu ()
@@ -135,82 +113,13 @@ public class PhotoView : EventBox {
 		return constraints_option_menu;
 	}
 
-	// Display.
-	private void HandlePixbufAreaUpdated (object sender, Gdk.AreaUpdatedArgs args)
-	{
-		// FIXME we should really only expose the updated region.
-		image_view.QueueDraw ();
-	}
-
-	private void UpdateImageView ()
-	{
-		if (CurrentPhotoValid ()) {
-			try {
-				Pixbuf old = image_view.Pixbuf;
-				
-				bool load_async = true;
-				if (load_async) {
-					image_view.Pixbuf = loader.Load (Query.Photos [current_photo].DefaultVersionPath);
-					loader.Loader.AreaUpdated += HandlePixbufAreaUpdated;
-					tag_view.Current = Query.Photos [current_photo];
-				} else {
-					image_view.Pixbuf = FSpot.PhotoLoader.Load (Query, current_photo);
-				}
-
-				if (old != null)
-					old.Dispose ();
-			} catch (GException ex) {
-				// FIXME
-				image_view.Pixbuf = null;
-			}
-		} else {
-			image_view.Pixbuf = null;
-		}
-
-		image_view.UnsetSelection ();
-		UpdateZoom ();
-	}
-
 	private uint restore_scrollbars_idle_id;
 
-	// FIXME need to remove this on dispose.
-	private bool IdleUpdateScrollbars ()
-	{
-		(image_view.Parent as ScrolledWindow).SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
-
-		restore_scrollbars_idle_id = 0;
-		return false;
-	}
-
-	private void UpdateZoom ()
-	{		
-		Pixbuf pixbuf = image_view.Pixbuf;
-
-		if (pixbuf == null)
-			return;
-
-		int available_width = image_view.Allocation.Width;
-		int available_height = image_view.Allocation.Height;
-
-		double zoom_to_fit = ZoomUtils.FitToScale ((uint) available_width, (uint) available_height,
-							   (uint) pixbuf.Width, (uint) pixbuf.Height, false);
-
-		double image_zoom = (MAX_ZOOM - zoom_to_fit) * Zoom + zoom_to_fit;
-		Console.WriteLine ("Zoom {2} zoom_to_fit {0} image_zoom {1}", zoom_to_fit, image_zoom, Zoom);
-
-		if (Math.Abs (Zoom) < double.Epsilon)
-			((ScrolledWindow) image_view.Parent).SetPolicy (PolicyType.Never, PolicyType.Never);
-
-		image_view.SetZoom (image_zoom, image_zoom);
-
-		if (Math.Abs (Zoom) < double.Epsilon && restore_scrollbars_idle_id == 0)
-			restore_scrollbars_idle_id = Idle.Add (new IdleHandler (IdleUpdateScrollbars));
-	}
 
 	private void UpdateButtonSensitivity ()
 	{
-		bool prev = CurrentPhotoValid () && current_photo > 0;
-		bool next = CurrentPhotoValid () && current_photo < query.Photos.Length -1;
+		bool prev = CurrentPhotoValid () && CurrentPhoto > 0;
+		bool next = CurrentPhotoValid () && CurrentPhoto < query.Photos.Length -1;
 
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
@@ -224,16 +133,16 @@ public class PhotoView : EventBox {
 			if (Query.Photos.Length == 0)
 				count_label.Text = String.Format ("{0} of {1}", 0, 0);
 			else 
-				count_label.Text = String.Format ("{0} of {1}", current_photo + 1, Query.Photos.Length);
+				count_label.Text = String.Format ("{0} of {1}", CurrentPhoto + 1, Query.Photos.Length);
 		}
 	}
 
 	private void UpdateDescriptionEntry ()
 	{
 		description_entry.Changed -= HandleDescriptionChanged;
-		if (Query.Photos.Length > 1 && current_photo < Query.Photos.Length) {
+		if (Query.Photos.Length > 1 && CurrentPhoto < Query.Photos.Length) {
 			description_entry.Sensitive = true;
-			description_entry.Text = Query.Photos[current_photo].Description;
+			description_entry.Text = Query.Photos[CurrentPhoto].Description;
 		} else {
 			description_entry.Sensitive = false;
 			description_entry.Text = "";
@@ -241,12 +150,11 @@ public class PhotoView : EventBox {
 		description_entry.Changed += HandleDescriptionChanged;
 	}    
 
-	public void Update ()
+	private void Update ()
 	{
 		if (UpdateStarted != null)
 			UpdateStarted (this);
 
-		UpdateImageView ();
 		UpdateButtonSensitivity ();
 		UpdateCountLabel ();
 		UpdateDescriptionEntry ();
@@ -260,18 +168,12 @@ public class PhotoView : EventBox {
 
 	private void DisplayNext ()
 	{
-		if (Query.Photos.Length > 1 && current_photo < Query.Photos.Length - 1) {
-			current_photo ++;
-			Update ();
-		}
+		photo_view.Next ();
 	}
 
 	private void DisplayPrevious ()
 	{
-		if (current_photo > 0) {
-			current_photo --;
-			Update ();
-		}
+		photo_view.Prev ();
 	}
 
 
@@ -285,54 +187,23 @@ public class PhotoView : EventBox {
 		}
 	}
 
-	private void HandleImageViewKeyPressEvent (object sender, KeyPressEventArgs args)
-	{
-		switch (args.Event.Key) {
-		case Gdk.Key.Page_Up:
-		case Gdk.Key.KP_Page_Up:
-			HandleDisplayPreviousButtonClicked (sender, null);
-			break;
-		case Gdk.Key.Page_Down:
-		case Gdk.Key.KP_Page_Down:
-			HandleDisplayNextButtonClicked (sender, null);
-			break;
-		default:
-			args.RetVal = false;
-			return;
-		}
-
-		args.RetVal = true;
-		return;
-	}
-
 	private void HandleDisplayNextButtonClicked (object sender, EventArgs args)
 	{
 		DisplayNext ();
-
-		if (PhotoChanged != null)
-			PhotoChanged (this);
 	}
 
 	private void HandleDisplayPreviousButtonClicked (object sender, EventArgs args)
 	{
 		DisplayPrevious ();
-
-		if (PhotoChanged != null)
-			PhotoChanged (this);
-	}
-
-	private void HandleImageViewSizeAllocated (object sender, SizeAllocatedArgs args)
-	{
-		UpdateZoom ();
 	}
 
 	private void HandleCropButtonClicked (object sender, EventArgs args)
 	{
 		int x, y, width, height;
-		if (! image_view.GetSelection (out x, out y, out width, out height))
+		if (! photo_view.GetSelection (out x, out y, out width, out height))
 			return;
 		
-		Pixbuf original_pixbuf = image_view.Pixbuf;
+		Pixbuf original_pixbuf = photo_view.Pixbuf;
 		if (original_pixbuf == null) {
 			Console.WriteLine ("No image");
 			return;
@@ -347,8 +218,8 @@ public class PhotoView : EventBox {
 
 		// FIXME the fact that the selection doesn't go away is a bug in ImageView, it should
 		// be fixed there.
-		image_view.Pixbuf = cropped_pixbuf;
-		image_view.UnsetSelection ();
+		photo_view.Pixbuf = cropped_pixbuf;
+		photo_view.UnsetSelection ();
 
 		System.Console.WriteLine ("Got here");
 
@@ -368,25 +239,23 @@ public class PhotoView : EventBox {
 			// FIXME error dialog.
 			Console.WriteLine ("error {0}", ex);
 		}
-
-		UpdateZoom ();
+		
+		photo_view.Fit = true;
 
 		if (PhotoChanged != null)
 			PhotoChanged (this);
 	}
 
 	private void HandleUnsharpButtonClicked (object sender, EventArgs args) {
-		//image_view.Pixbuf = PixbufUtils.UnsharpMask (image_view.Pixbuf, 6, 2, 0);
-		//image_view.Pixbuf = PixbufUtils.ColorCorrect (image_view.Pixbuf);
-		new ColorDialog (Query, current_photo, image_view.Pixbuf);
+		new FSpot.ColorDialog (Query, CurrentPhoto);
 	}	
 
 	private void HandleDescriptionChanged (object sender, EventArgs args) {
 		if (!CurrentPhotoValid ())
 			return;
 
-		Query.Photos[current_photo].Description = description_entry.Text;
-		Query.Commit (current_photo);
+		Query.Photos[CurrentPhoto].Description = description_entry.Text;
+		Query.Commit (CurrentPhoto);
 	}
 
 	// Constructor.
@@ -435,9 +304,23 @@ public class PhotoView : EventBox {
 		constraints[9].XyRatio = 1.0;
 	}
 
-	public PhotoView (PhotoStore photo_store)
+	private void HandlePhotoChanged (FSpot.PhotoImageView view)
+	{
+		Update ();
+
+		if (CurrentPhotoValid ()) 
+			tag_view.Current = query.Photos [CurrentPhoto];
+		else 
+			tag_view.Current = null;
+
+		if (this.PhotoChanged != null)
+			PhotoChanged (this);
+	}
+
+	public PhotoView (FSpot.PhotoQuery query, PhotoStore photo_store)
 		: base ()
 	{
+		this.query = query;
 		this.photo_store = photo_store;
 
 		BorderWidth = 3;
@@ -455,27 +338,26 @@ public class PhotoView : EventBox {
 
 		frame.Add (inner_vbox);
 		
-		image_view = new FSpot.ImageView ();
+		photo_view = new FSpot.PhotoImageView (query);
+		photo_view.PhotoChanged += HandlePhotoChanged;
 
-		ScrolledWindow image_view_scrolled = new ScrolledWindow (null, null);
+		ScrolledWindow photo_view_scrolled = new ScrolledWindow (null, null);
 
 		Gdk.Color color = eventbox.Style.Background (Gtk.StateType.Normal);
 		color.Red = (ushort) (color.Red / 2);
 		color.Blue = (ushort) (color.Blue / 2);
 		color.Green = (ushort) (color.Green / 2);
 
-		image_view.ModifyBg (Gtk.StateType.Normal, color);
+		photo_view.ModifyBg (Gtk.StateType.Normal, color);
 		eventbox.ModifyBg (Gtk.StateType.Normal, color);
-		image_view_scrolled.ModifyBg (Gtk.StateType.Normal, color);
+		photo_view_scrolled.ModifyBg (Gtk.StateType.Normal, color);
 
-		image_view_scrolled.SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
-		image_view_scrolled.ShadowType = ShadowType.None;
-		image_view_scrolled.Add (image_view);
-		image_view_scrolled.ButtonPressEvent += HandleButtonPressEvent;
-		image_view.SizeAllocated += new SizeAllocatedHandler (HandleImageViewSizeAllocated);
-		image_view.AddEvents ((int) EventMask.KeyPressMask);
-		image_view.KeyPressEvent += new KeyPressEventHandler (HandleImageViewKeyPressEvent);
-		inner_vbox.PackStart (image_view_scrolled, true, true, 0);
+		photo_view_scrolled.SetPolicy (PolicyType.Automatic, PolicyType.Automatic);
+		photo_view_scrolled.ShadowType = ShadowType.None;
+		photo_view_scrolled.Add (photo_view);
+		photo_view_scrolled.ButtonPressEvent += HandleButtonPressEvent;
+		photo_view.AddEvents ((int) EventMask.KeyPressMask);
+		inner_vbox.PackStart (photo_view_scrolled, true, true, 0);
 		
 		HBox inner_hbox = new HBox (false, 2);
 		inner_hbox.BorderWidth = 6;
@@ -531,10 +413,5 @@ public class PhotoView : EventBox {
 		vbox.ShowAll ();
 	}
 
-	public PhotoView (PhotoQuery query, PhotoStore photo_store)
-		: this (photo_store)
-	{
-		Query = query;
-	}
 }
 
