@@ -4,6 +4,17 @@ using System.Runtime.InteropServices;
 using System;
 using System.IO;
 
+public enum PixbufOrientation {
+	TopLeft = 1,
+	TopRight = 2,
+	BottomRight = 3,
+	BottomLeft = 4,
+	LeftTop = 5,
+	RightTop = 6,
+	RightBottom = 7,
+	LeftBottom = 8
+}
+
 class PixbufUtils {
 
 	public static Pixbuf ErrorPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
@@ -69,6 +80,7 @@ class PixbufUtils {
 	public class AspectLoader : Gdk.PixbufLoader {
 		int max_width;
 		int max_height;
+		PixbufOrientation orientation;
 
 		// FIXME: this should be a property
 		public bool ScaleAlongLongestEdge = false;
@@ -83,6 +95,18 @@ class PixbufUtils {
 		private void HandleSizePrepared (object obj, SizePreparedArgs args)
 		{
 			double scale;
+			switch (orientation) {
+			case PixbufOrientation.LeftTop:
+			case PixbufOrientation.LeftBottom:
+			case PixbufOrientation.RightTop:
+			case PixbufOrientation.RightBottom:	
+				int tmp = max_width;
+				max_width = max_height;
+				max_height = tmp;
+				break;
+			default:
+				break;
+			}
 
 			if (ScaleAlongLongestEdge) {
 				if (args.Width > args.Height)
@@ -104,6 +128,8 @@ class PixbufUtils {
 		public Pixbuf LoadFromFile (string path)
 		{
 			FileStream fs;
+
+			orientation = GetOrientation (path);
 			try {
 				fs = File.OpenRead (path);
 			} catch (Exception e) {
@@ -117,7 +143,9 @@ class PixbufUtils {
 				;
 			
 			this.Close ();
-			return this.Pixbuf;
+			Gdk.Pixbuf rotated = TransformOrientation (this.Pixbuf, orientation, true);
+			
+			return rotated;
 		}
 	}
 
@@ -242,7 +270,7 @@ class PixbufUtils {
 					  double hue, double saturation, int src_color, int dest_color)
 	{
 		Pixbuf adjusted = new Pixbuf (Colorspace.Rgb, src.HasAlpha, 8, src.Width, src.Height);
-		ColorAdjust (src, adjusted, brightness, contrast, hue, saturation, src_color, dest_color);
+		PixbufUtils.ColorAdjust (src, adjusted, brightness, contrast, hue, saturation, src_color, dest_color);
 		return adjusted;
 	}
 
@@ -251,7 +279,6 @@ class PixbufUtils {
 		return buf.HasAlpha ? Cms.Format.Rgba8 : Cms.Format.Rgb8;
 	}
 
-
 	public static unsafe void ColorAdjust (Pixbuf src, Pixbuf dest, 
 					       double brightness, double contrast,
 					       double hue, double saturation, 
@@ -272,6 +299,15 @@ class PixbufUtils {
 							 PixbufCmsFormat (dest),
 							 Cms.Intent.Perceptual, 0x0100);
 
+		ColorAdjust (src, dest, trans);
+
+		trans.Dispose ();
+		srgb.Dispose ();
+		bchsw.Dispose ();
+	}
+
+	public static unsafe void ColorAdjust (Gdk.Pixbuf src, Gdk.Pixbuf dest, Cms.Transform trans)
+	{
 		int width = src.Width;
 		byte * srcpix  = (byte *) src.Pixels;
 		byte * destpix = (byte *) dest.Pixels;
@@ -282,9 +318,134 @@ class PixbufUtils {
 				     (uint)width);
 		}
 		
-		trans.Dispose ();
-		srgb.Dispose ();
-		bchsw.Dispose ();
+	}
+
+	public static PixbufOrientation GetOrientation (string path)
+	{
+		ExifData exif = new ExifData (path);
+		byte [] value = exif.LookupData (ExifTag.Orientation);
+		PixbufOrientation orientation = PixbufOrientation.TopLeft;
+
+		if (value != null) {
+			System.Console.WriteLine ("len = {0} val [0] = {1} string = {2}", value.Length, 
+						  value[0], exif.LookupString (ExifTag.Orientation));
+			orientation = (PixbufOrientation)value [0];
+		}
+
+		return orientation;
+	}
+
+	public static Gdk.Pixbuf TransformOrientation (Gdk.Pixbuf src, PixbufOrientation orientation, bool copy_data)
+	{
+		Gdk.Pixbuf pixbuf;
+		switch (orientation) {
+		case PixbufOrientation.LeftTop:
+		case PixbufOrientation.LeftBottom:
+		case PixbufOrientation.RightTop:
+		case PixbufOrientation.RightBottom:	
+			pixbuf = new Gdk.Pixbuf (src.Colorspace, src.HasAlpha, 
+						 src.BitsPerSample,
+						 src.Height, src.Width);
+			break;
+		case PixbufOrientation.TopRight:
+		case PixbufOrientation.BottomRight:
+		case PixbufOrientation.BottomLeft:
+			pixbuf = new Gdk.Pixbuf (src.Colorspace, src.HasAlpha, 
+						 src.BitsPerSample,
+						 src.Width, src.Height);
+			break;
+		default:
+			pixbuf = src;
+			break;
+		}
+
+		if (copy_data && src != pixbuf) 
+			TransformAndCopy (src, pixbuf, orientation, new Gdk.Rectangle (0, 0, src.Width, src.Height));
+
+		return pixbuf;
+	}
+
+	public static Gdk.Pixbuf TransformOrientation (Gdk.Pixbuf src, PixbufOrientation orientation)
+	{
+		return TransformOrientation (src, orientation, true);
+	}
+
+	public static Gdk.Rectangle TransformAndCopy (Gdk.Pixbuf src, Gdk.Pixbuf dest, PixbufOrientation orientation, Gdk.Rectangle args)
+	{
+		Gdk.Rectangle area = args;
+		Gdk.Pixbuf region;
+		Gdk.Pixbuf altered;
+		
+		switch (orientation) {
+		case PixbufOrientation.LeftBottom:
+			area.X = args.Y;
+			area.Y = src.Width - args.X - args.Width;
+			area.Width = args.Height;
+			area.Height = args.Width;
+			
+			region = new Gdk.Pixbuf (src, args.X, args.Y,
+						 args.Width, args.Height);
+			altered = PixbufUtils.Rotate90 (region, true);
+
+			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
+
+			region.Dispose ();
+			altered.Dispose ();
+			break;
+		case PixbufOrientation.RightTop:
+			area.X = src.Height - args.Y - args.Height;
+			area.Y = args.X;
+			area.Width = args.Height;
+			area.Height = args.Width;
+			
+			region = new Gdk.Pixbuf (src, args.X, args.Y,
+						 args.Width, args.Height);
+			altered = PixbufUtils.Rotate90 (region, false);
+
+			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
+
+			region.Dispose ();
+			altered.Dispose ();			
+			break;
+		case PixbufOrientation.BottomRight:
+			area.X = src.Width - args.X - args.Width;
+			area.Y = src.Height - args.Y - args.Height;
+
+			region = new Gdk.Pixbuf (src, args.X, args.Y,
+						 args.Width, args.Height);
+			altered = PixbufUtils.Mirror (region, true, true);
+			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
+
+			region.Dispose ();
+			altered.Dispose ();
+			break;
+		case PixbufOrientation.TopRight:
+			area.X = src.Width - args.X - args.Width;
+
+			region = new Gdk.Pixbuf (src, args.X, args.Y,
+						 args.Width, args.Height);
+			altered = PixbufUtils.Mirror (region, true, false);
+			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
+
+			region.Dispose ();
+			altered.Dispose ();
+			break;
+		case PixbufOrientation.BottomLeft:
+			area.Y = src.Height - args.Y - args.Height;
+
+			region = new Gdk.Pixbuf (src, args.X, args.Y,
+						 args.Width, args.Height);
+			altered = PixbufUtils.Mirror (region, false, true);
+			altered.CopyArea (0, 0, altered.Width, altered.Height, dest, area.X, area.Y);
+
+			region.Dispose ();
+			altered.Dispose ();
+			break;
+		default:
+			src.CopyArea (area.X, area.Y, area.Width, area.Height, dest, area.X, area.Y);
+			return area;
+		}
+		return area;
 	}
 
 	// Bindings from libf.
