@@ -26,7 +26,7 @@ namespace FSpot {
 		private ThumbnailCache preview_cache = new ThumbnailCache (50);
 
 		private int item;
-		public int Item {
+		new public int Item {
 			get {
 				return item;
 			}
@@ -171,10 +171,12 @@ namespace FSpot {
 			case Gdk.Key.v:
 				ShowHistogram = false;
 				UpdateItem ();
+				args.RetVal = true;
 				break;
 			case Gdk.Key.V:
 				ShowHistogram = true;
 				UpdateItem ();
+				args.RetVal = true;
 				break;
 			}
 		}
