@@ -77,7 +77,7 @@ namespace FSpot {
 			Gnome.PrintContext ctx = print_job.Context;
 
 			foreach (Photo photo in photos) {
-				Gnome.Print.Beginpage (ctx, "F-Spot" + photo.DefaultVersionPath);				
+				ctx.BeginPage ("F-Spot" + photo.DefaultVersionPath);				
 
 				Gdk.Pixbuf image = FSpot.PhotoLoader.Load (photo);
 
@@ -93,22 +93,22 @@ namespace FSpot {
 				double scale = System.Math.Min (width / image.Width, 
 								height / image.Height);
 
-				Gnome.Print.Gsave (ctx);
+				ctx.SaveGraphicState ();
 
 				if (rotate) {
-					Gnome.Print.Rotate (ctx, 90);
-					Gnome.Print.Translate (ctx, 0, -page_width);
+					ctx.Rotate (90);
+					ctx.Translate (0, -page_width);
 				}
 				
-				Gnome.Print.Translate (ctx,
-						       (width - image.Width * scale) / 2.0,
-						       (height - image.Height * scale) / 2.0);
+				ctx.Translate (
+					       (width - image.Width * scale) / 2.0,
+					       (height - image.Height * scale) / 2.0);
 				
-				Gnome.Print.Scale (ctx, image.Width * scale, image.Height * scale);
+				ctx.Scale (image.Width * scale, image.Height * scale);
 				Gnome.Print.Pixbuf (ctx, image);
-				Gnome.Print.Grestore (ctx);
+				ctx.RestoreGraphicState ();
 
-				Gnome.Print.Showpage (ctx);
+				ctx.ShowPage ();
 				image.Dispose ();
 			}
 			
@@ -116,6 +116,11 @@ namespace FSpot {
 		}
 
 		private void HandleConfigureClicked (object sender, System.EventArgs args)
+		{
+			RunGnomePrintDialog ();
+		}
+
+		private void RunGnomePrintDialog ()
 		{
 			Gnome.PrintDialog gnome_dialog = new Gnome.PrintDialog (print_job, "Print Images", 0);
 			int response = gnome_dialog.Run ();
@@ -135,13 +140,15 @@ namespace FSpot {
 		{
 			this.photos = photos;
 
+#if ENABLE_CUSTOM_PRINT
 			Glade.XML xml = new Glade.XML (null, "f-spot.glade", "print_dialog", null);
 			xml.Autoconnect (this);
+#endif
 
 			print_job = new Gnome.PrintJob (Gnome.PrintConfig.Default ());
 
 			Render ();
-
+#if ENABLE_CUSTOM_PRINT
 			int response = print_dialog.Run ();
 			
 			switch (response) {
@@ -150,6 +157,9 @@ namespace FSpot {
 				break;
 			}
 			print_dialog.Destroy ();
+#else
+			RunGnomePrintDialog ();
+#endif
 		}
 	}
 }
