@@ -53,7 +53,7 @@ public class TagCommands {
 			Menu menu = new Menu ();
 
 			if (categories.Count == 0) {
-				MenuItem item = new MenuItem ("(No categories)");
+				MenuItem item = new MenuItem (Mono.Posix.Catalog.GetString ("(No categories)"));
 				category_option_menu.Sensitive = false;
 				menu.Append (item);
 			} else {
@@ -99,7 +99,7 @@ public class TagCommands {
 				already_in_use_label.Markup = "";
 			} else if (TagNameExistsInCategory (tag_name_entry.Text, tag_store.RootCategory)) {
 				ok_button.Sensitive = false;
-				already_in_use_label.Markup = "<small>This name is already in use</small>";
+				already_in_use_label.Markup = "<small>" + Mono.Posix.Catalog.GetString ("This name is already in use") + "</small>";
 			} else {
 				ok_button.Sensitive = true;
 				already_in_use_label.Markup = "";
@@ -120,12 +120,12 @@ public class TagCommands {
 
 			switch (type) {
 			case TagType.Tag:
-				create_tag_dialog.Title = "Create New Tag";
-				prompt_label.Text = "Name of new tag:";
+				create_tag_dialog.Title = Mono.Posix.Catalog.GetString ("Create New Tag");
+				prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of new tag:");
 				break;
 			case TagType.Category:
-				create_tag_dialog.Title = "Create New Category";
-				prompt_label.Text = "Name of new category:";
+				create_tag_dialog.Title = Mono.Posix.Catalog.GetString ("Create New Category");
+				prompt_label.Text = Mono.Posix.Catalog.GetString ("Name of new category:");
 				break;
 			}
 
@@ -228,7 +228,7 @@ public class TagCommands {
 			} else if (TagNameExistsInCategory (name, db.Tags.RootCategory)
 				   && name != orig_name) {
 				ok_button.Sensitive = false;
-				already_in_use_label.Markup = "<small>This name is already in use</small>";
+				already_in_use_label.Markup = "<small>" + Mono.Posix.Catalog.GetString ("This name is already in use") + "</small>";
 			} else {
 				ok_button.Sensitive = true;
 				already_in_use_label.Markup = "";
@@ -303,11 +303,11 @@ public class TagCommands {
 			edit_tag_dialog.DefaultResponse = ResponseType.Ok;
 
 			if (t is Category) {
-				edit_tag_dialog.Title = "Edit Category";
-				prompt_label.Text = "Category name:";
+				edit_tag_dialog.Title = Mono.Posix.Catalog.GetString ("Edit Category");
+				prompt_label.Text = Mono.Posix.Catalog.GetString ("Category name:");
 			} else {
-				edit_tag_dialog.Title = "Edit Tag";
-				prompt_label.Text = "Tag name:";
+				edit_tag_dialog.Title = Mono.Posix.Catalog.GetString ("Edit Tag");
+				prompt_label.Text = Mono.Posix.Catalog.GetString ("Tag name:");
 			}
 
 			orig_name = last_valid_name = t.Name;
@@ -349,9 +349,9 @@ public class TagCommands {
 
 	public class EditIcon {
 		Db db;
-		Photo [] photos;
 		Gtk.Window parent_window;
-		FSpot.ImageView image_view;
+		FSpot.PhotoQuery query;
+		FSpot.PhotoImageView image_view;
 
 		[Glade.Widget]
 		Dialog edit_icon_dialog;
@@ -368,27 +368,12 @@ public class TagCommands {
 		[Glade.Widget]
 		SpinButton photo_spin_button;
 
-		int current_item = -1;
 		public int CurrentItem {
 			get {
-				return current_item;
+				return image_view.CurrentPhoto;
 			}
 			set {
-				if (value != current_item) {
-					current_item = value;
-					photo_label.Text = String.Format ("Photo {0} of {1}", 
-									  current_item + 1, photos.Length);
-
-					Gdk.Pixbuf old = image_view.Pixbuf;
-					image_view.Pixbuf = PixbufUtils.LoadAtMaxSize (photos [value].DefaultVersionPath,
-										       image_view.Parent.Allocation.Width,
-										       image_view.Parent.Allocation.Height);
-
-					photo_spin_button.Value = (double)current_item + 1;
-
-					if (old != null)
-						old.Dispose ();
-				}
+				image_view.CurrentPhoto = value;
 			}
 		}
 		
@@ -396,8 +381,7 @@ public class TagCommands {
 		{
 			int value = photo_spin_button.ValueAsInt - 1;
 			
-			if (value != current_item)
-				CurrentItem = value;
+			image_view.CurrentPhoto = value;
 		}
 
 		private void HandleSelectionChanged ()
@@ -416,6 +400,14 @@ public class TagCommands {
 			}
 		}
 
+		public void HandlePhotoChanged (FSpot.PhotoImageView sender)
+		{
+			photo_label.Text = String.Format (Mono.Posix.Catalog.GetString ("Photo {0} of {1}"), 
+							  image_view.CurrentPhoto + 1, query.Photos.Length);
+
+			photo_spin_button.Value = image_view.CurrentPhoto + 1;
+		}
+
 		public bool Execute (Tag t)
 		{
 			Glade.XML xml = new Glade.XML (null, "f-spot.glade", "edit_icon_dialog", null);
@@ -424,35 +416,37 @@ public class TagCommands {
 			edit_icon_dialog.DefaultResponse = ResponseType.Ok;
 
 			if (t is Category) {
-				edit_icon_dialog.Title = String.Format ("Edit icon For category {0}", t.Name);
+				edit_icon_dialog.Title = String.Format (Mono.Posix.Catalog.GetString ("Edit icon For category {0}"), t.Name);
 			} else {
-				edit_icon_dialog.Title = String.Format ("Edit icon for tag {0}", t.Name);
+				edit_icon_dialog.Title = String.Format (Mono.Posix.Catalog.GetString ("Edit icon for tag {0}"), t.Name);
 			}
 
 			preview_image.Pixbuf = t.Icon;
-			image_view = new FSpot.ImageView ();
+
+			query = new FSpot.PhotoQuery (db.Photos);
+			query.Tags = new Tag [] { t, db.Tags.Hidden };
+
+			image_view = new FSpot.PhotoImageView (query);
 			image_view.SelectionXyRatio = 1.0;
-			image_view.Show ();
 			image_view.SelectionChanged += HandleSelectionChanged;
+			image_view.PhotoChanged += HandlePhotoChanged;
 
 			photo_scrolled_window.Add (image_view);
 
-			Tag [] tags = new Tag [] { t, db.Tags.Hidden };
-			photos = db.Photos.Query (tags);
-			
-			if (photos.Length > 0) {
+			if (query.Photos.Length > 0) {
 				photo_spin_button.Wrap = true;
 				photo_spin_button.Adjustment.Lower = 1.0;
-				photo_spin_button.Adjustment.Upper = (double)photos.Length;
+				photo_spin_button.Adjustment.Upper = (double)query.Photos.Length;
 				photo_spin_button.Adjustment.StepIncrement = 1.0;
 				photo_spin_button.ValueChanged += HandleSpinButtonChanged;
 				
-				CurrentItem = 0;
+				image_view.CurrentPhoto = 0;
 			} else {
 				photo_spin_button.Sensitive = false;
 				photo_spin_button.Value = 0.0;
 			}			
 			
+			image_view.Show ();
 
 			ResponseType response = (ResponseType) edit_icon_dialog.Run ();
 			bool success = false;
